package ru.tandemservice.uniclient.unimirea_code.base.bo.MailService.ui.MilitaryDocument;

import org.apache.commons.io.FileUtils;
import org.apache.tapestry.request.IUploadFile;
import org.tandemframework.caf.ui.UIPresenter;
import org.tandemframework.core.CoreStringUtils;
import org.tandemframework.core.component.Bind;
import org.tandemframework.core.component.Input;
import org.tandemframework.core.entity.OrderDirection;
import org.tandemframework.core.runtime.ApplicationRuntime;
import org.tandemframework.hibsupport.DataAccessServices;
import org.tandemframework.hibsupport.builder.MQBuilder;
import org.tandemframework.hibsupport.builder.expression.MQExpression;
import org.tandemframework.hibsupport.dql.DQLExecutionContext;
import org.tandemframework.hibsupport.dql.DQLSelectBuilder;
import org.tandemframework.shared.commonbase.remoteDocument.bo.RemoteDocument.RemoteDocumentManager;
import org.tandemframework.shared.commonbase.remoteDocument.bo.RemoteDocument.logic.RemoteDocumentMeta;
import org.tandemframework.tapsupport.component.selection.BaseSingleSelectModel;
import org.tandemframework.tapsupport.component.selection.ISingleSelectModel;
import org.tandemframework.tapsupport.component.selection.ListResult;
import ru.tandemservice.movestudent.dao.MoveStudentDao;
import ru.tandemservice.uni.entity.base.Student;
import ru.tandemservice.uni.entity.base.StudentCustomState;
import ru.tandemservice.uni.entity.catalog.StudentCustomStateCI;
import ru.tandemservice.uniclient.unimirea_code.base.bo.MailService.utils.MailServiceUtils;
import ru.tandemservice.uniclient.unimirea_code.clientUtils.UnimireaProjectUtils;
import ru.tandemservice.uniclient.unimirea_code.entity.ClientRemoteFile;
import ru.tandemservice.uniclient.unimirea_code.entity.DigitalSignatureMirea;
import ru.tandemservice.uniclient.unimirea_code.entity.MilitaryDocument;
import ru.tandemservice.uniclient.unimirea_code.entity.catalog.MailServiceConfig;

import javax.mail.MessagingException;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import static org.tandemframework.hibsupport.dql.DQLExpressions.*;

@Input({@Bind(key = "studentId", binding = "studentId")})
public class MailServiceMilitaryDocumentUI extends UIPresenter {

    private Long studentId;
    private IUploadFile document;
    private IUploadFile documentSig;
    private String message;

    private StudentCustomStateCI stateCI;

    private MilitaryDocument militaryDocument;

    public MailServiceMilitaryDocumentUI() {
    }

    @Override
    public void onComponentRefresh() {
    }

    public void onClickApply() {
        List<String> tempPaths = new ArrayList<>();

        try {
            MailServiceConfig config = DataAccessServices.dao().get(MailServiceConfig.class, MailServiceConfig.userCode(), "military");

            MailServiceConfig newConfig = new MailServiceConfig();
            newConfig.setSmtpHost(config.getSmtpHost());
            newConfig.setSmtpPort(config.getSmtpPort());
            newConfig.setEmail(config.getEmail());
            newConfig.setPassword(config.getPassword());
            newConfig.setHeader(config.getHeader());
            newConfig.setBody(config.getBody());

            if (config != null && militaryDocument.getMilitaryDocumentType().getIndex() == 4) {
                newConfig.setHeader("Рассылка справки об обучении для военного комиссариата");
                newConfig.setBody(
                        "<p>Добрый день!</p>" +
                                "<p>Управление мобилизационной подготовки РТУ МИРЭА направляет Вам Справку об обучении (Приложение № 32 п.3 к Инструкции, утв. приказом Минобороны РФ от 02 октября 2007 г. № 400)</p>" +
                                "<p>Для контроля за наличием у студента права на отсрочку на сайте РТУ МИРЭА создан Реестр студентов-призывников: <a href=\"https://www.mirea.ru/recruits/\" >https://www.mirea.ru/recruits/</a></p>" +
                                "<p>Перечень электронных адресов военных комиссариатов: <a href=\"https://student.mirea.ru/mobilization-preparation/\" >https://student.mirea.ru/mobilization-preparation/</a></p>" +
                                "<p>С Уважением, Управление мобилизационной подготовки РТУ МИРЭА.</p>"
                );
            }

            String studentEmail = UnimireaProjectUtils.getStudentEmail(studentId);

            if (config != null && studentEmail != null) {
                if (addDigitalSignature()) {
                    tempPaths = getTempPaths();
                    MailServiceUtils.sendHtmlMailWithAttachments(newConfig, studentEmail, tempPaths);
                    updateStatus();
                    for (String tempPath : tempPaths) {
                        new File(tempPath).delete();
                    }
                    deactivate();
                } else {
                    getSupport().error("Произошла ошибка при сохранении ЭЦП");
                }
            } else {
                getSupport().error("Произошла ошибка при отправке");
            }

        } catch (IOException | MessagingException e) {
            getSupport().error("Произошла ошибка при отправке");
            e.printStackTrace();
            for (String tempPath : tempPaths) {
                new File(tempPath).delete();
            }
        }

    }

    /**
     * Метод возвращает список путей до прикрепленных файлов
     *
     * @return Список путей до прикрепленных файлов
     */
    private List<String> getTempPaths() {
        List<String> result = new ArrayList<>();
        String basePath = ApplicationRuntime.getProperty("temp_doc.path");
        try {

            ClientRemoteFile clientRemoteFile = DataAccessServices.dao().get(ClientRemoteFile.class, ClientRemoteFile.entityId(), militaryDocument.getId());
            String typeName = militaryDocument.getMilitaryDocumentType().getTitle();
            String filePath = basePath + "/" + CoreStringUtils.transliterate(typeName + "_" + militaryDocument.getStudent().getFio()) + ".pdf";
            File file = new File(filePath);
            FileUtils.writeByteArrayToFile(file, clientRemoteFile.getFileDto().getContent());
            result.add(filePath);


            if (documentSig != null) {
                String fileSigPath = basePath + "/" + documentSig.getFileName();
                File fileSig = new File(fileSigPath);
                documentSig.write(fileSig);
                result.add(fileSigPath);
            }
        } catch (IOException e) {
            e.printStackTrace();
            for (String tempPath : result) {
                new File(tempPath).delete();
            }
        }

        return result;

    }

    /**
     * Метод возвращает список дополнительных статусов для студента
     *
     * @param type Тип
     * @return Список дополнительных статусов для студента
     */
    private List<StudentCustomStateCI> getCustomStatuses(String type) {
        return DataAccessServices.dao().getList(
                new DQLSelectBuilder().fromEntity(StudentCustomStateCI.class, "sci")
                        .where(like(property(StudentCustomStateCI.shortTitle().fromAlias("sci")), value(type + " %")))
                        .order(property(StudentCustomStateCI.shortTitle().fromAlias("sci")), OrderDirection.desc)
        );
    }

    /**
     * Метод выполняет добавление нового дополнительного статуса для студента П2П3
     */
    private void updateStatus() {
        if (stateCI != null) {
            if (!UnimireaProjectUtils.checkStudentCustomStateCI(studentId, stateCI)) {
                StudentCustomState studentCustomState = new StudentCustomState();
                studentCustomState.setCustomState(stateCI);
                studentCustomState.setStudent(DataAccessServices.dao().get(studentId));
                DataAccessServices.dao().save(studentCustomState);
            } else {
                getSupport().error("У студента уже имеется дополнительный статус " + message);
            }
        } else {
            getSupport().error("Произошла ошибка в установке дополнительного статуса" + message + " для студента. Попробуйте установить вручную.");
        }

    }

    /**
     * Метод сохраняет ЭЦП документа в хранилище
     *
     * @return Статус выполнения
     */
    private boolean addDigitalSignature() {
        List<DigitalSignatureMirea> list = DataAccessServices.dao().getList(
                new DQLSelectBuilder().fromEntity(DigitalSignatureMirea.class, "d")
                        .where(eq(property(DigitalSignatureMirea.entityId().fromAlias("d")), value(militaryDocument.getId())))
        );
        militaryDocument.setDigitalSignatureMirea(null);
        DataAccessServices.dao().saveOrUpdate(militaryDocument);
        for (DigitalSignatureMirea digitalSignatureMirea : list) {
            DataAccessServices.dao().delete(digitalSignatureMirea);
        }

        DigitalSignatureMirea digitalSignatureMirea = new DigitalSignatureMirea();
        digitalSignatureMirea.setEntityId(militaryDocument.getId());

        switch (militaryDocument.getMilitaryDocumentType().getIndex()) {
            case 1: {
                digitalSignatureMirea.setType("militaryDocumentP2P3");
                break;
            }
            case 2: {
                digitalSignatureMirea.setType("militaryDocumentPR9");
                break;
            }
            case 4: {
                digitalSignatureMirea.setType("militaryDocumentP32P2");
                break;
            }
        }
        DataAccessServices.dao().saveOrUpdate(digitalSignatureMirea);

        if (documentSig != null) {
            RemoteDocumentMeta meta = digitalSignatureMirea.getRemoteMeta().setDefaultFileContent(documentSig);
            RemoteDocumentManager.instance().utilDao().createOrUpdateDefaultRemoteDocumentWithOwnerUpdate(meta);
        } else {
            return false;
        }

        digitalSignatureMirea = DataAccessServices.dao().get(DigitalSignatureMirea.class, DigitalSignatureMirea.entityId(), militaryDocument.getId());

        militaryDocument.setDigitalSignatureMirea(digitalSignatureMirea);
        DataAccessServices.dao().saveOrUpdate(militaryDocument);
        return true;
    }

    /**
     * При выборе документа подгружается последний соответствующий Доп.статус
     */
    public void onChooseDocument() {
        String type = null;

        switch (militaryDocument.getMilitaryDocumentType().getIndex()) {
            case 1: {
                type = "П2П3";
                break;
            }
            case 2: {
                type = "Пр9";
                break;
            }
            case 4: {
                type = "П32П2";
                break;
            }
        }

        List<StudentCustomStateCI> customStateCISLocal = getCustomStatuses(type);

        if (customStateCISLocal != null && !customStateCISLocal.isEmpty() && customStateCISLocal.get(0) != null) {
            this.stateCI = customStateCISLocal.get(0);
            this.message = stateCI.getShortTitle();
        } else {
            this.message = "";
            getSupport().error("Не найдены дополнительные статусы " + type);
        }

    }


    // Документ
    private ISingleSelectModel _militaryModel = new BaseSingleSelectModel("titleForMail") {
        @Override
        public Object getValue(Object o) {
            MQBuilder builder = new MQBuilder(MilitaryDocument.ENTITY_CLASS, "e");
            builder.add(MQExpression.eq("e", "id", o));
            List resultList = builder.getResultList(_uiSupport.getSession());
            return resultList.isEmpty() ? null : resultList.iterator().next();
        }

        @Override
        public ListResult findValues(String s) {
            DQLSelectBuilder builder = new DQLSelectBuilder();
            builder.fromEntity(MilitaryDocument.class, "e");
            builder.order(property(MilitaryDocument.militaryDocumentType().title().fromAlias("e")));
            builder.where(eq(property(MilitaryDocument.student().id().fromAlias("e")), value(studentId)));

            builder.where(in(property(MilitaryDocument.militaryDocumentType().code().fromAlias("e")), Arrays.asList("md.1", "md.2", "md.4")));

            if (!Objects.equals(s, "")) {
                builder.where(likeUpper(MilitaryDocument.content().filename().fromAlias("e"), value("%" + s + "%")));
            }

            int count = ((Number) builder.createCountStatement(new DQLExecutionContext(_uiSupport.getSession())).uniqueResult()).intValue();
            if (count > MoveStudentDao.MAX_ROWS)
                builder.top(MoveStudentDao.MAX_ROWS);

            return new ListResult<>(builder.createStatement(_uiSupport.getSession()).<MilitaryDocument>list(), count);
        }
    };

    public ISingleSelectModel getMilitaryModel() {
        return _militaryModel;
    }

    public void setMilitaryModel(ISingleSelectModel _yearModel) {
        this._militaryModel = _militaryModel;
    }

    public Long getStudentId() {
        return studentId;
    }

    public void setStudentId(Long studentId) {
        this.studentId = studentId;
    }

    public IUploadFile getDocument() {
        return document;
    }

    public void setDocument(IUploadFile document) {
        this.document = document;
    }

    public IUploadFile getDocumentSig() {
        return documentSig;
    }

    public void setDocumentSig(IUploadFile documentSig) {
        this.documentSig = documentSig;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public MilitaryDocument getMilitaryDocument() {
        return militaryDocument;
    }

    public void setMilitaryDocument(MilitaryDocument militaryDocument) {
        this.militaryDocument = militaryDocument;
    }
}