package ru.tandemservice.uniclient.unimirea_code.base.bo.MailService.utils;

import com.ibm.icu.text.Transliterator;
import org.tandemframework.core.CoreStringUtils;
import org.tandemframework.core.exception.ApplicationException;
import ru.tandemservice.uniclient.unimirea_code.clientUtils.UnimireaProjectUtils;
import ru.tandemservice.uniclient.unimirea_code.entity.catalog.MailServiceConfig;

import javax.mail.*;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Properties;

public class MailServiceUtils {


    /**
     * Метод возвращает сессию для отправки электронных писем
     *
     * @return возвращает сессию для отправки электронных писем
     */
    public static Session getMailSession() {
        Properties prop = new Properties();
        prop.put("mail.smtp.auth", true);
        prop.put("mail.smtp.starttls.enable", "false");
        prop.put("mail.smtp.host", "smtp.mirea.ru");
        prop.put("mail.smtp.port", "25");

        return Session.getInstance(prop, new Authenticator() {
            @Override
            protected PasswordAuthentication getPasswordAuthentication() {
                return new PasswordAuthentication("email@mirea.ru", "123456");
            }
        });
    }

    /**
     * Метод возвращает сессию для отправки электронных писем
     *
     * @param host     SMTP хост
     * @param port     SMTP порт
     * @param email    email
     * @param password пароль
     * @return возвращает сессию для отправки электронных писем
     */
    public static Session getMailSession(String host, String port, String email, String password) {
        Properties prop = new Properties();
        prop.put("mail.smtp.auth", true);
        prop.put("mail.smtp.starttls.enable", "false");
        prop.put("mail.smtp.host", host);
        prop.put("mail.smtp.port", port);

        return Session.getInstance(prop, new Authenticator() {
            @Override
            protected PasswordAuthentication getPasswordAuthentication() {
                return new PasswordAuthentication(email, password);
            }
        });
    }


    /**
     * Метод выполняет отправку электронного письма с текстом на указанный email
     *
     * @param fromEmail Электронная почта отправителя
     * @param toEmail   Электронная почта получателя
     * @param subject   Тема письма
     * @param mailText  Содержание письма
     * @throws MessagingException Возможно исключение
     */
    public static void sendMail(Session session, String fromEmail, String toEmail, String subject, String mailText) throws MessagingException {
        if (UnimireaProjectUtils.checkDebugMode()) {
            throw new ApplicationException("\"Method didn't execute, because \"debug.mode = true\"");
        }
        Message message = new MimeMessage(session);
        message.setFrom(new InternetAddress(fromEmail));

        toEmail = toEmail.trim().replaceAll("(\\r|\\n)", "");
        message.setRecipients(Message.RecipientType.TO, InternetAddress.parse(toEmail));
        message.setSubject(subject);

        MimeBodyPart mimeBodyPart = new MimeBodyPart();
        mimeBodyPart.setText(mailText, "UTF-8");

        Multipart multipart = new MimeMultipart();
        multipart.addBodyPart(mimeBodyPart);

        message.setContent(multipart);

        Transport.send(message);

    }

    /**
     * Метод выполняет отправку электронного письма с текстом на указанный email
     *
     * @param config Настройки для электронной почты
     * @param email  Электронная почта получателя
     * @throws MessagingException Возможно исключение
     */
    public static void sendMail(MailServiceConfig config, String email) throws MessagingException {
        if (UnimireaProjectUtils.checkDebugMode()) {
            throw new ApplicationException("\"Method didn't execute, because \"debug.mode = true\"");
        }
        sendMail(
                getMailSession(config.getSmtpHost(), config.getSmtpPort(), config.getEmail(), config.getPassword()),
                config.getEmail(),
                email,
                config.getHeader(),
                config.getBody()
        );

    }


    /**
     * Метод выполняет отправку электронного письма в виде html на указанный email
     *
     * @param fromEmail Электронная почта отправителя
     * @param toEmail   Электронная почта получателя
     * @param subject   Тема письма
     * @param html      html код
     * @throws MessagingException Возможно исключение
     */
    public static void sendHtmlMail(Session session, String fromEmail, String toEmail, String subject, String html) throws MessagingException {
        if (UnimireaProjectUtils.checkDebugMode()) {
            throw new ApplicationException("\"Method didn't execute, because \"debug.mode = true\"");
        }
        MimeMessage message = new MimeMessage(session);
        message.setFrom(new InternetAddress(fromEmail));

        toEmail = toEmail.trim().replaceAll("(\\r|\\n)", "");
        message.setRecipients(Message.RecipientType.TO, InternetAddress.parse(toEmail));
        message.setSubject(subject);

        message.setContent(html, "text/html; charset=utf-8");

        Transport.send(message);

    }

    /**
     * Метод выполняет отправку электронного письма в виде html на указанный email
     *
     * @param config Настройки для электронной почты
     * @param email  Электронная почта получателя
     * @throws MessagingException Возможно исключение
     */
    public static void sendHtmlMail(MailServiceConfig config, String email) throws MessagingException {
        if (UnimireaProjectUtils.checkDebugMode()) {
            throw new ApplicationException("\"Method didn't execute, because \"debug.mode = true\"");
        }
        sendHtmlMail(
                getMailSession(config.getSmtpHost(), config.getSmtpPort(), config.getEmail(), config.getPassword()),
                config.getEmail(),
                email,
                config.getHeader(),
                config.getBody()
        );

    }


    /**
     * Метод выполняет отправку электронного письма в виде html с приложенными файлами на указанный email
     *
     * @param session         Сессия
     * @param fromEmail       Электронная почта отправителя
     * @param toEmail         Электронная почта получателя
     * @param subject         Тема письма
     * @param html            html код
     * @param attachmentPaths Пути до прикладываемых файлов
     * @throws MessagingException Возможно исключение
     * @throws IOException        Возможна ошибка ввода/вывода
     */
    public static void sendHtmlMailWithAttachments(Session session, String fromEmail, String toEmail, String subject, String html, List<String> attachmentPaths) throws MessagingException, IOException {
        if (UnimireaProjectUtils.checkDebugMode()) {
            throw new ApplicationException("\"Method didn't execute, because \"debug.mode = true\"");
        }
        Message message = new MimeMessage(session);

        message.setFrom(new InternetAddress(fromEmail));

        toEmail = toEmail.trim().replaceAll("(\\r|\\n)", "");
        message.setRecipients(Message.RecipientType.TO, InternetAddress.parse(toEmail));
    
        message.setSubject(subject);

        MimeBodyPart messageBodyPart = new MimeBodyPart(); // Создание  текствовой части письма
        messageBodyPart.setContent(html, "text/html; charset=utf-8");

        Multipart multipart = new MimeMultipart(); // Создание структуры для содержимого письма
        multipart.addBodyPart(messageBodyPart);

        // Добавление к письму файлов
        if (attachmentPaths != null) {
            for (String attachmentPath : attachmentPaths) {
                File temp = new File(attachmentPath);

                MimeBodyPart attachPart = new MimeBodyPart();
                attachPart.attachFile(temp);
                attachPart.setFileName(
                        temp.getName().matches(".+[а-я].+")
                                ? CoreStringUtils.transliterate(temp.getName())
                                : temp.getName()
                );
                multipart.addBodyPart(attachPart);
            }
        }

        message.setContent(multipart);

        Transport.send(message);

    }

    /**
     * Метод выполняет отправку электронного письма в виде html с приложенными файлами на указанный email
     *
     * @param config          Настройки для электронной почты
     * @param email           Электронная почта получателя
     * @param attachmentPaths Пути до прикладываемых файлов
     * @throws IOException        Возможна ошибка ввода/вывода
     * @throws MessagingException Возможно исключение
     */
    public static void sendHtmlMailWithAttachments(MailServiceConfig config, String email, List<String> attachmentPaths) throws IOException, MessagingException {
        if (UnimireaProjectUtils.checkDebugMode()) {
            throw new ApplicationException("\"Method didn't execute, because \"debug.mode = true\"");
        }
        sendHtmlMailWithAttachments(
                getMailSession(config.getSmtpHost(), config.getSmtpPort(), config.getEmail(), config.getPassword()),
                config.getEmail(),
                email,
                config.getHeader(),
                config.getBody(),
                attachmentPaths
        );
    }

    /**
     * Метод производит транслитерацию на латиницу
     *
     * @param text Текст для транслитерации
     * @return Транслитерированный текст
     */
    public static String toLatinTrans(String text) {
        Transliterator transliterator = Transliterator.getInstance("Cyrillic-Latin");
        return transliterator.transliterate(text);
    }


}
