package ru.tandemservice.uniclient.unimirea_code.base.ext.Person.ui.Tab;

import org.tandemframework.caf.ui.IUIPresenter;
import org.tandemframework.caf.ui.addon.UIAddon;
import org.tandemframework.core.component.BusinessComponentUtils;
import org.tandemframework.core.entity.OrderDirection;
import org.tandemframework.core.exception.ApplicationException;
import org.tandemframework.core.view.formatter.DateFormatter;
import org.tandemframework.core.view.list.column.ActionColumn;
import org.tandemframework.core.view.list.column.SimpleColumn;
import org.tandemframework.core.view.list.source.DynamicListDataSource;
import org.tandemframework.hibsupport.DataAccessServices;
import org.tandemframework.hibsupport.dql.DQLExpressions;
import org.tandemframework.hibsupport.dql.DQLSelectBuilder;
import org.tandemframework.shared.commonbase.base.bo.Common.CommonManager;
import org.tandemframework.shared.commonbase.base.entity.DatabaseFile;
import org.tandemframework.shared.commonbase.base.util.CommonBaseRenderer;
import org.tandemframework.shared.commonbase.base.util.CommonBaseUtil;
import org.tandemframework.shared.commonbase.remoteDocument.dto.RemoteDocumentDTO;
import org.tandemframework.shared.employeebase.base.entity.EmployeePost;
import org.tandemframework.shared.person.base.bo.Person.ui.BenefitAddEdit.PersonBenefitAddEdit;
import org.tandemframework.shared.person.base.bo.Person.ui.Tab.PersonTabUI;
import org.tandemframework.shared.person.base.entity.Person;
import org.tandemframework.shared.person.base.entity.PersonBenefit;
import ru.tandemservice.uni.UniUtils;
import ru.tandemservice.uni.entity.base.Student;
import ru.tandemservice.uni.entity.base.StudentCustomState;
import ru.tandemservice.uni.entity.base.StudentDocument;
import ru.tandemservice.uni.entity.catalog.StudentCustomStateCI;
import ru.tandemservice.uni.entity.catalog.UniScriptItem;
import ru.tandemservice.uniclient.unimirea_code.base.bo.MailService.ui.MilitaryDocument.MailServiceMilitaryDocument;
import ru.tandemservice.uniclient.unimirea_code.base.bo.MilitaryDocument.ui.Add.MilitaryDocumentAdd;
import ru.tandemservice.uniclient.unimirea_code.entity.*;
import ru.tandemservice.uniclient.unimirea_code.entity.catalog.codes.UniScriptItemCodes;
import ru.tandemservice.uniclient.unimirea_code.entity.gen.InfoHealthAdditionGen;

import java.util.ArrayList;
import java.util.List;

import static org.tandemframework.hibsupport.dql.DQLExpressions.*;


/**
 * Created by 22 on 20.07.2017.
 */

public class PersonTabExtUI extends UIAddon {
    public static final String STUDENT = "student";
    private Person person;
    private Student _student;
    protected PersonTabUI _presenter;

    public PersonTabExtUI(IUIPresenter presenter, String name, String componentId) {
        super(presenter, name, componentId);
        _presenter = (PersonTabUI) this.getPresenter();
    }

    public void onComponentRefresh() {
        person = _presenter.getPerson();
        Object secSot = _presenter.getPersonRoleModel().getSecuredObject();
        if (secSot instanceof Student) {
            _student = (Student) secSot;    // если заходим через Tab Student
        }
        this.prepareDocumentDataSource();   //  документы воинского учета
        
    }

    // ----------- документы воинского учета ---------------
    /*В PersonTabUI есть ISecureRoleContext personRoleModel, который для карточки студента в getSecuredObject должен содержать студента.
    Вы можете в своем аддоне PersonTabExtUI взять из презентера этот personRoleModel.getSecuredObject, проверить, что там instanceof Student,
    и на основании этого отрисовывать кнопку, брать id студента и прочее.
            */
    private DynamicListDataSource<MilitaryDocument> documentDS;

    public DynamicListDataSource<MilitaryDocument> getDocumentDS() {
        return this.documentDS;
    }

    public void setDocumentDS(final DynamicListDataSource<MilitaryDocument> documentDS) {
        this.documentDS = documentDS;
    }

    public boolean isAddButtonActive() {
        Object secSot = _presenter.getPersonRoleModel().getSecuredObject();
        return (secSot instanceof Student);
    }

    private void prepareDocumentDataSource() {
        final DynamicListDataSource<MilitaryDocument> dataSource = new DynamicListDataSource<>(_presenter.getConfig().getBusinessComponent(), component -> {
            DQLSelectBuilder builder = new DQLSelectBuilder().fromEntity(MilitaryDocument.class, "b");
            if (_student != null)
                builder.where(eq(property(MilitaryDocument.student().fromAlias("b")), value(_student)));
            else
                builder.where(eq(property(MilitaryDocument.student().person().fromAlias("b")), value(_presenter.getPerson())));
            List<MilitaryDocument> list = builder.createStatement(getSession()).list();
            getDocumentDS().setTotalSize(list.size());
            getDocumentDS().setCountRow(list.size());
            UniUtils.createPage(getDocumentDS(), list);
        });
//  DATE_FORMATTER_WITH_TIME
        dataSource.addColumn(new SimpleColumn("Дата формирования", MilitaryDocument.formingDate(), DateFormatter.DATE_FORMATTER_WITH_TIME).setOrderable(false).setClickable(false));
        dataSource.addColumn(new SimpleColumn("Номер", MilitaryDocument.fullNumber()));
        dataSource.addColumn(new SimpleColumn("Тип документа", MilitaryDocument.militaryDocumentType().title().s()).setOrderable(false));
        dataSource.addColumn(new SimpleColumn("Исполнитель", MilitaryDocument.executor().s()).setOrderable(false));
        dataSource.addColumn(CommonBaseUtil.getPrintColumn("unimirea_codePersonTabExtUI:onClickDocumentFile", "Печатать документ"));
        dataSource.addColumn(CommonBaseUtil.getPrintPdfColumn("unimirea_codePersonTabExtUI:onClickDocumentFilePDF", "Печатать документ PDF"));
        dataSource.addColumn(CommonBaseUtil.getPrintColumn("unimirea_codePersonTabExtUI:onClickDownloadDigitalSignature", "Печатать ЭЦП"));
        dataSource.addColumn(new ActionColumn("Удалить", ActionColumn.DELETE, "unimirea_codePersonTabExtUI:onClickDeleteDocument", "Удалить документ № {0}?",
                MilitaryDocument.fullNumber()));
        //  .setDisabledProperty(PersonAcademicDegree.P_PRINTING_DISABLED).setPermissionKey(this._secModel.getPermission("printFilePersonScienceDegree")));
        setDocumentDS(dataSource);
    }

    public void onClickDocumentFile() {
        MilitaryDocument militaryDocument = DataAccessServices.dao().get(getListenerParameterAsLong());
        if (militaryDocument.getContent().getContent() == null)
            throw new ApplicationException("Файл печатной формы пуст.");

        String fileName = "";
        if (militaryDocument.getRtfFileName() == null) {
            String typeName = militaryDocument.getMilitaryDocumentType().getTitle();
            fileName = typeName + "_" + _student.getFio() + ".rtf";
        } else {
            fileName = militaryDocument.getRtfFileName();
        }

        BusinessComponentUtils.downloadDocument(new CommonBaseRenderer().fileName(fileName).document(militaryDocument.getContent()).rtf(), true);

        if (militaryDocument.getMilitaryDocumentType().getCode().equals("md.2")) {
            updateStatus(militaryDocument);
        }
    }

    public void onClickDocumentFilePDF() {
        MilitaryDocument militaryDocument = DataAccessServices.dao().get(getListenerParameterAsLong());
        ClientRemoteFile clientRemoteFile = DataAccessServices.dao().get(ClientRemoteFile.class, ClientRemoteFile.entityId(), militaryDocument.getId());
        if (clientRemoteFile != null && clientRemoteFile.getFileDto() != null) {
            RemoteDocumentDTO documentDTO = clientRemoteFile.getFileDto();
            String FILENAME = documentDTO.getFileName();
            BusinessComponentUtils.downloadDocument(new CommonBaseRenderer().fileName(FILENAME).document(documentDTO.getContent()), true);
        } else {
            throw new ApplicationException("Файл PDF отсутствует.");
        }
    }

    public void onClickDownloadDigitalSignature() {
        MilitaryDocument militaryDocument = DataAccessServices.dao().get(getListenerParameterAsLong());
        if (militaryDocument.getDigitalSignatureMirea() != null && militaryDocument.getDigitalSignatureMirea().getContent() != null) {
            RemoteDocumentDTO documentDTO = militaryDocument.getDigitalSignatureMirea().getFileDto();
            String typeName = militaryDocument.getMilitaryDocumentType().getTitle();
//            String FILENAME = "ЭЦП " + typeName + "_" + _student.getFio() + ".sig";
            String FILENAME = documentDTO.getFileName();
            BusinessComponentUtils.downloadDocument(new CommonBaseRenderer().fileName(FILENAME).document(documentDTO.getContent()), true);
        } else {
            throw new ApplicationException("Файл ЭЦП отсутствует.");
        }
    }

    public void onClickDeleteDocument() {
        MilitaryDocument militaryDocument = DataAccessServices.dao().get(getListenerParameterAsLong());
        DatabaseFile content = militaryDocument.getContent();
        DataAccessServices.dao().delete(militaryDocument);
        if (null != content) {
            StudentDocument studentDocument = DataAccessServices.dao().get(StudentDocument.class, StudentDocument.L_CONTENT, content);
            if (null != studentDocument) {
                DataAccessServices.dao().delete(studentDocument);
            }
            DataAccessServices.dao().delete(content);
        }

        // Удаление ЭЦП
        DigitalSignatureMirea digitalSignatureMirea = DataAccessServices.dao().get(DigitalSignatureMirea.class, DigitalSignatureMirea.entityId(), militaryDocument.getId());
        if (digitalSignatureMirea != null) {
            DataAccessServices.dao().delete(digitalSignatureMirea);
        }

        // Удаление PDF
        ClientRemoteFile clientRemoteFile = DataAccessServices.dao().get(ClientRemoteFile.class, ClientRemoteFile.entityId(), militaryDocument.getId());
        if (clientRemoteFile != null) {
            DataAccessServices.dao().delete(clientRemoteFile);
        }

    }

    public void onClickAddDocument() {
        getActivationBuilder().asRegionDialog(MilitaryDocumentAdd.class).parameter("studentId", _student.getId()).activate();
    }

    public void onClickPrintFormT2() {
        UniScriptItem scriptItem = DataAccessServices.dao().getByCode(UniScriptItem.class, UniScriptItemCodes.STUDENT_FORM_T2);
        String executorFio = _presenter.getUserContext().getPrincipalContext().getFio();
        String executorPost = new DQLSelectBuilder().fromEntity(EmployeePost.class, "ep")
                .column(property(EmployeePost.postRelation().postBoundedWithQGandQL().title().fromAlias("ep")))
                .where(
                        eq(
                                property(EmployeePost.principal().fromAlias("ep")),
                                value(_presenter.getUserContext().getPrincipalContext().getPrincipal())
                        )
                )
                .where(eq(property(EmployeePost.postStatus().code().fromAlias("ep")), value("1")))
                .createStatement(_presenter.getSupport().getSession())
                .uniqueResult();

        if (_student != null) {
            CommonManager.instance().scriptDao().getScriptResultAndDownloadIt(scriptItem,
                    "student", _student,
                    "executorFIO", executorFio,
                    "executorPost", executorPost
            );
        }
        /*
        UniScriptItem scriptItem = DataAccessServices.dao().getByCode(UniScriptItem.class, UniScriptItemCodes.STUDENT_FORM_T2);
        if (_student != null)
            CommonManager.instance().scriptDao().getScriptResultAndDownloadIt(scriptItem, _student);
         */
    }

    public void onClickSendDocument() {
        getActivationBuilder().asRegionDialog(MailServiceMilitaryDocument.class).parameter("studentId", _student.getId()).activate();
    }

    /**
     * Метод выполняет добавление нового дополнительного статуса для студента Пр9
     */
    private void updateStatus(MilitaryDocument militaryDocument) {
//        StudentCustomStateCI stateCI = DataAccessServices.dao().get(StudentCustomStateCI.class, StudentCustomStateCI.code(), "147");

        StudentCustomStateCI stateCI;

        List<StudentCustomStateCI> stateCISList = DataAccessServices.dao().getList(
                new DQLSelectBuilder().fromEntity(StudentCustomStateCI.class, "s")
                        .where(like(property(StudentCustomStateCI.title().fromAlias("s")), value("Пр9 %")))
                        .order(property(StudentCustomStateCI.title().fromAlias("s")), OrderDirection.desc)
        );
        if (!stateCISList.isEmpty()) {
            stateCI = stateCISList.get(0);
        } else {
            throw new ApplicationException("Не удалось установить доп. статус Пр9");
        }
        boolean notExistCustomStatus = DataAccessServices.dao().getList(
                new DQLSelectBuilder().fromEntity(StudentCustomState.class, "scs")
                        .where(eq(property(StudentCustomState.student().id().fromAlias("scs")), value(militaryDocument.getStudent().getId())))
                        .where(eq(property(StudentCustomState.customState().fromAlias("scs")), value(stateCI)))
        ).isEmpty();
        if (notExistCustomStatus) {
            StudentCustomState studentCustomState = new StudentCustomState();
            studentCustomState.setCustomState(stateCI);
            studentCustomState.setStudent(militaryDocument.getStudent());
            DataAccessServices.dao().save(studentCustomState);
        } else {
            throw new ApplicationException("Не удалось установить доп. статус Пр9. Возможно он уже прикреплен у студента.");
        }

    }


}