package ru.tandemservice.uniclient.unimirea_code.clientUtils;

import jxl.CellView;
import jxl.write.WritableSheet;
import org.apache.tapestry.request.IUploadFile;
import org.tandemframework.core.exception.ApplicationException;
import org.tandemframework.core.runtime.ApplicationRuntime;
import org.tandemframework.hibsupport.DataAccessServices;
import org.tandemframework.hibsupport.dql.DQL;
import org.tandemframework.hibsupport.dql.DQLJoinType;
import org.tandemframework.hibsupport.dql.DQLSelectBuilder;
import org.tandemframework.shared.commonbase.catalog.entity.codes.InflectorVariantCodes;
import org.tandemframework.shared.commonbase.remoteDocument.bo.RemoteDocument.RemoteDocumentManager;
import org.tandemframework.shared.commonbase.remoteDocument.bo.RemoteDocument.logic.RemoteDocumentMeta;
import org.tandemframework.shared.commonbase.utils.ooffice.OpenOfficeServiceUtil;
import org.tandemframework.shared.employeebase.base.entity.EmployeePost;
import org.tandemframework.shared.employeebase.catalog.entity.Post;
import ru.tandemservice.uni.entity.base.Group;
import ru.tandemservice.uni.entity.base.OrderData;
import ru.tandemservice.uni.entity.base.Student;
import ru.tandemservice.uni.entity.base.StudentCustomState;
import ru.tandemservice.uni.entity.catalog.StudentCustomStateCI;
import ru.tandemservice.uniclient.unimirea_code.entity.DigitalSignatureMirea;
import ru.tandemservice.unieductr.base.entity.EduCtrEducationPromise;
import ru.tandemservice.unisession.entity.document.SessionDocumentSlot;
import ru.tandemservice.unisession.entity.document.SessionStudentGradeBookDocument;
import ru.tandemservice.unisession.entity.mark.SessionMark;

import java.io.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

import static org.tandemframework.hibsupport.dql.DQLExpressions.*;

/**
 * Утилиты модуля unimirea_code.
 * ВАЖНО: ПРИ ДОБАВЛЕНИИ НОВОГО МЕТОДА НЕОБХОДИМО СОЗДАТЬ JAVADOC И УКАЗАТЬ АВТОРА С ПОМОЩЬЮ АННОТАЦИИ @author
 *
 * @author Алексеев К.С.
 */
public class UnimireaProjectUtils {

    /**
     * Метод возвращает предполагаемый год окончания обучения заданного студента
     *
     * @param studentId Id студента
     * @return Возвращает предполагаемый год окончания обучения заданного студента
     * @author Покровская Т.М.
     */
    public static int getFinishYear(Long studentId) {
        Student student;
        if (studentId != null) {
            student = DataAccessServices.dao().get(studentId);
        } else {
            return -1;
        }
        //-------------------
        if (student == null) {
            return -1;
        }
        int endYear = 0;
        // TODO Костыль с учетом отчисленных и архивных студентами
        // для закончивших с дипломом (неархивнх и архивных)
        if (student.getFinishYear() != null) {
            endYear = student.getFinishYear();
            return endYear;
        }
        //для отчсленных
        if (student.getStatus().getCode().equals("8")) {
            OrderData orderData = DataAccessServices.dao().get(OrderData.class, OrderData.student(), student);
            if (orderData != null && orderData.getExcludeOrderDate() != null) {
                Date excDate = orderData.getExcludeOrderDate();
                Calendar cale = Calendar.getInstance();
                cale.setTime(excDate);
                endYear = cale.get(Calendar.YEAR);
                return endYear;
            } else {
                return -1;
            }
        }
        int course = student.getCourse().getIntValue(); // текущий курс студента
        Group group = student.getGroup();
        int courseGroup = group.getCourse().getIntValue();
        String groupTitle = group.getTitle();
        int poi = groupTitle.indexOf("-", 6);
        int yearStart = 0;
        if (poi == -1) {
            yearStart = group.getStartEducationYear().getIntValue();
        } else {
            yearStart = Integer.valueOf("20" + groupTitle.substring(poi + 1, poi + 3));           // год группы
        }
        if (groupTitle == "РААО-01-19") {            // номер группы и год начала не соответствуют году приема - это аспиранты
            yearStart = 2020;
        }
        int programYears = 0;
        int programMonths = 0;
        boolean acad = student.getStatus().isUsedAsAcademicLeave();           // в академке
        if (student.getDevelopPeriodAuto() != null) {
            programYears = student.getDevelopPeriodAuto().getNumberOfYears();
            programMonths = student.getDevelopPeriodAuto().getNumberOfMonths();
            endYear = yearStart + programYears;
            if ((course < courseGroup) && acad) {        // в академке - группа у студента осталась, а курс у группы уже другой
                endYear = endYear + courseGroup - course;
            }
            if (programMonths > 4) {
                endYear++;
            }
            return endYear;
        } else {
            return -1;
        }
    }

//    public static int getFinishYear(Long studentId) {
//        Student student;
//        if (studentId != null) {
//            student = DataAccessServices.dao().get(studentId);
//        } else {
//            return -1;
//        }
//        //-------------------
//        if (student == null) {
//            return -1;
//        }
//        int endYear = 0;
//        // TODO Костыль с учетом отчисленных и архивных студентами
//        // для закончивших с дипломом (неархивнх и архивных)
//        if (student.getFinishYear() != null) {
//            endYear = student.getFinishYear();
//            return endYear;
//        }
//        //для отчсленных
//        if (student.getStatus().getCode().equals("8")) {
//            OrderData orderData = DataAccessServices.dao().get(OrderData.class, OrderData.student(), student);
//            if (orderData != null) {
//                Date excDate = orderData.getExcludeOrderDate();
//                Calendar cale = Calendar.getInstance();
//                cale.setTime(excDate);
//                endYear = cale.get(Calendar.YEAR);
//                return endYear;
//            }
//        }
//        int course = student.getCourse().getIntValue(); // текущий курс студента
//        Group group = student.getGroup();
//        int courseGroup = group.getCourse().getIntValue();
//        String groupTitle = group.getTitle();
//        int poi = groupTitle.indexOf("-", 6);
//        int yearStart = 0;
//        if (poi == -1) {
//            yearStart = group.getStartEducationYear().getIntValue();
//        } else {
//            yearStart = Integer.valueOf("20" + groupTitle.substring(poi + 1, poi + 3));           // год группы
//        }
//        if (groupTitle == "РААО-01-19")            // номер группы и год начала не соответствуют году приема - это аспиранты
//            yearStart = 2020;
//        int programYears = 0;
//        int programMonths = 0;
//        boolean acad = student.getStatus().isUsedAsAcademicLeave();           // в академке
//        if (student.getDevelopPeriodAuto() != null) {
//            programYears = student.getDevelopPeriodAuto().getNumberOfYears();
//            programMonths = student.getDevelopPeriodAuto().getNumberOfMonths();
//            endYear = yearStart + programYears;
//            if ((course < courseGroup) && acad) {        // в академке - группа у студента осталась, а курс у группы уже другой
//                endYear = endYear + courseGroup - course;
//            }
//            if (programMonths > 4)
//                endYear++;
//            return endYear;
//        } else
//            return -1;
//    }

    /**
     * Метод возвращает предполагаемую дату окончания обучения заданного студента
     *
     * @param studentId Id студента
     * @return Возвращает предполагаемую дату окончания обучения заданного студента
     * @author Покровская Т.М.
     */
    public static Date getFinishDate(Long studentId) {
        Student student;
        if (studentId != null) {
            student = DataAccessServices.dao().get(studentId);
        } else {
            return null;
        }
        if (student == null) {
            return null;
        }
        int endYear = 0;
        Date endDate = null;
        int course = student.getCourse().getIntValue(); // текущий курс студента
        Group group = student.getGroup();
        int courseGroup;
        try {
            courseGroup = group.getCourse().getIntValue();
        } catch (NullPointerException e) {
            throw new ApplicationException("Невозможно вычислить предполагаемую дату окончания обучения, поскольку у студента отсутствует группа.");
        }
        String groupTitle = group.getTitle();
        int poi = groupTitle.indexOf("-", 6);
        int yearStart = 0;
        if (poi == -1) {
            yearStart = group.getStartEducationYear().getIntValue();
        } else {
            yearStart = Integer.valueOf("20" + groupTitle.substring(poi + 1, poi + 3));           // год группы
        }
        if (groupTitle == "РААО-01-19")            // номер группы и год начала не соответствуют году приема - это аспиранты
            yearStart = 2020;
        int programYears = 0;
        int programMonths = 0;
        boolean acad = student.getStatus().isUsedAsAcademicLeave();           // в академке
        if (student.getDevelopPeriodAuto() != null) {
            programYears = student.getDevelopPeriodAuto().getNumberOfYears();
            programMonths = student.getDevelopPeriodAuto().getNumberOfMonths();
            endYear = yearStart + programYears;
            if ((course < courseGroup) && acad) {        // в академке - группа у студента осталась, а курс у группы уже другой
                endYear = endYear + courseGroup - course;
            }
            if (programMonths > 4)
                endYear++;
            int gg = endYear;
            Calendar calendar = Calendar.getInstance();
            int currMonth = calendar.get(Calendar.MONTH);
            int currYear = calendar.get(Calendar.YEAR);
            //        if (endYear<=currYear && course==courseGroup)                                 // остались в старой группе, а вся группа уже закончила-академкка или после академки
            //            endYear=currYear+(currYear-endYear)+1;
            String formUnit = student.getEducationOrgUnit().getFormativeOrgUnit().getTitle();
            int monthStart = 9;
            String formCode = student.getEducationOrgUnit().getDevelopForm().getCode();
            if ((student.getEducationOrgUnit().getDevelopForm().getCode()).equals("2") || formUnit.contains("спирантур") || formUnit.contains("Ставропол") && (student.getEducationOrgUnit().getDevelopForm().getCode()).equals("3"))
                monthStart = 10;

            // заочная форма или аспиранты, или Ставрополь и очно-заочная форма
            int finishMonth = monthStart + (programMonths - 1) < 13 ? monthStart + (programMonths - 1) : monthStart + (programMonths - 1) - 12;
            Calendar c = Calendar.getInstance();
            c.set(Calendar.YEAR, endYear);
            c.set(Calendar.MONTH, finishMonth);
            c.set(Calendar.DAY_OF_MONTH, 1);
            c.add(Calendar.DAY_OF_MONTH, -1);
            int finishDay = c.get(Calendar.DAY_OF_MONTH);   // последний день месяца
            c.set(Calendar.MONTH, finishMonth - 1);
            c.set(Calendar.DAY_OF_MONTH, finishDay);
            endDate = c.getTime();
            String sss = "sss";
        }
        return endDate;
    }

    /**
     * Метод возвращает средний балл студента. Считаются итоговые оценки
     * <p>- Без учета зачетов;</p>
     * <p>- Включая курсовые и практики.</p>
     *
     * @param studentId Id студента
     * @return Возвращает средний балл студента
     * @author Алексеев К.С.
     */
    public static double getAvgStudentMark(Long studentId) {
        BigDecimal result = new BigDecimal(0);
        List<String> list = DataAccessServices.dao().getList(
                new DQLSelectBuilder()
                        .fromEntity(SessionStudentGradeBookDocument.class, "doc")
                        .column(property(SessionMark.cachedMarkValue().code().fromAlias("sm")))
                        .joinEntity("doc", DQLJoinType.inner, SessionDocumentSlot.class, "ds",
                                eq(property(SessionDocumentSlot.document().id().fromAlias("ds")), property(SessionStudentGradeBookDocument.id().fromAlias("doc")))
                        )
                        .joinEntity("ds", DQLJoinType.inner, SessionMark.class, "sm",
                                eq(property(SessionMark.slot().id().fromAlias("sm")), property(SessionDocumentSlot.id().fromAlias("ds")))
                        )
                        .where(eq(property(SessionStudentGradeBookDocument.student().id().fromAlias("doc")), value(studentId)))
                        .where(eq(property(SessionDocumentSlot.inSession().fromAlias("ds")), value(false)))
                        .where(in(property(SessionMark.cachedMarkValue().code().fromAlias("sm")), Arrays.asList("scale5.2", "scale5.3", "scale5.4", "scale5.5")))
        );

        for (String mark : list) {
            switch (mark) {
                case "scale5.2": {
                    result = result.add(new BigDecimal(2));
                    break;
                }
                case "scale5.3": {
                    result = result.add(new BigDecimal(3));
                    break;
                }
                case "scale5.4": {
                    result = result.add(new BigDecimal(4));
                    break;
                }
                case "scale5.5": {
                    result = result.add(new BigDecimal(5));
                    break;
                }
            }
        }

        return list.size() > 0 ? result.divide(new BigDecimal(list.size()), 2, RoundingMode.HALF_UP).doubleValue() : 0;

    }

    /**
     * Метод проверяет есть ли у студента определенный дополнительный статус
     *
     * @param stateCI Дополнительный статус
     * @return Имется - true; Не имеется - false
     * @author Алексеев К.С.
     */
    public static boolean checkStudentCustomStateCI(Long studentId, StudentCustomStateCI stateCI) {
        return !(DataAccessServices.dao().getList(
                new DQLSelectBuilder().fromEntity(StudentCustomState.class, "scs")
                        .where(eq(property(StudentCustomState.student().id().fromAlias("scs")), value(studentId)))
                        .where(eq(property(StudentCustomState.customState().fromAlias("scs")), value(stateCI)))
        ).isEmpty());

    }

    /**
     * Метод сохраняет ЭЦП документа в хранилище
     *
     * @param entityId    Id Сущности к которой привязана ЭЦП
     * @param type        Тип ЭЦП
     * @param documentSig Файл ЭЦП
     * @return Статус выполнения
     * @author Алексеев К.С.
     */
    public static boolean addDigitalSignature(Long entityId, String type, IUploadFile documentSig) {
        DigitalSignatureMirea digitalSignatureMirea = new DigitalSignatureMirea();
        digitalSignatureMirea.setEntityId(entityId);
        digitalSignatureMirea.setType(type);
        DataAccessServices.dao().saveOrUpdate(digitalSignatureMirea);

        if (documentSig != null) {
            RemoteDocumentMeta meta = digitalSignatureMirea.getRemoteMeta().setDefaultFileContent(documentSig);
            RemoteDocumentManager.instance().utilDao().createOrUpdateDefaultRemoteDocumentWithOwnerUpdate(meta);
        } else {
            return false;
        }

        return true;
    }

    /**
     * Метод сохраняет ЭЦП документа в хранилище
     *
     * @param entityId    Id Сущности к которой привязана ЭЦП
     * @param type        Тип ЭЦП
     * @param fileName    Название файла
     * @param fileContent Файл ЭЦП
     * @return Статус выполнения
     * @author Алексеев К.С.
     */
    public static boolean addDigitalSignature(Long entityId, String type, String fileName, byte[] fileContent) {
        DigitalSignatureMirea digitalSignatureMirea = new DigitalSignatureMirea();
        digitalSignatureMirea.setEntityId(entityId);
        digitalSignatureMirea.setType(type);
        DataAccessServices.dao().saveOrUpdate(digitalSignatureMirea);

        if (fileName != null && !fileName.isEmpty() && fileContent != null) {
            RemoteDocumentMeta meta = digitalSignatureMirea.getRemoteMeta().setDefaultFileContent(fileName, fileContent);
            RemoteDocumentManager.instance().utilDao().createOrUpdateDefaultRemoteDocumentWithOwnerUpdate(meta);
        } else {
            return false;
        }

        return true;
    }

    /**
     * Метод устанавливает размер ячеек для excel листа
     *
     * @param sheet  Лист excel
     * @param widths Значения ширины ячеек
     * @author Алексеев К.С.
     */
    public static void setCellSize(WritableSheet sheet, ArrayList<Integer> widths) {
        for (int i = 0; i < widths.size(); i++) {
            CellView cellView = new CellView();
            cellView.setSize(widths.get(i) * 256);
            sheet.setColumnView(i, cellView);
        }
    }

    /**
     * Метод производит преобразование rtf в pdf
     *
     * @param content  Rtf документ
     * @param filename Название оригинального документа ( пример.rtf )
     * @return Путь до преобразованного в pdf файла
     * @author Алексеев К.С.
     */
    public static String convertRTF2PDF(byte[] content, String filename) throws IOException {
        String path = ApplicationRuntime.getProperty("temp_doc.path");

        String name = path + "/" + filename.replaceFirst("\\.rtf$", ".pdf");

        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        OpenOfficeServiceUtil.convertRtfToPdf(new ByteArrayInputStream(content), baos);

        try (OutputStream outputStream = new FileOutputStream(name)) {
            baos.writeTo(outputStream);
        }

        return name;
    }

    /**
     * Количество семестров для  обр.программы печати договоров...
     */
    public static String semNumber(Long eduPromiseId) {
        EduCtrEducationPromise eduPromise = DataAccessServices.dao().get(eduPromiseId);
        int god = eduPromise.getEducationOrgUnit().getDevelopPeriod().getNumberOfYears();
        int mes = eduPromise.getEducationOrgUnit().getDevelopPeriod().getNumberOfMonths();
        int se = mes > 0 ? god * 2 + 1 : god * 2;
        if (mes > 7)
            se++;
        return String.valueOf(se);
    }

    /**
     * Метод печатает dql запрос в консольный вывод
     *
     * @param dql DQL
     * @author Алексеев К.С.
     */
    public static void printDQL(DQLSelectBuilder dql) {
        System.out.println(DQL.toString(dql.buildQuery()));
    }

    /**
     * Метод печатает dql запрос с меткой в консольный вывод. ( Например: MYDQL: select s.id from Student s )
     *
     * @param tag Метка
     * @param dql DQL
     * @author Алексеев К.С.
     */
    public static void printDQL(String tag, DQLSelectBuilder dql) {
        System.out.println(tag.toUpperCase() + ": " + DQL.toString(dql.buildQuery()));
    }

    /**
     * Метод возвращает название должности в заданном падеже. Если падеж не найден, то возвращается строка для ручного заполнения
     *
     * @param employeePost Сотрудник
     * @param caseTitle    Падеж (константа: InflectorVariantCodes)
     * @return Возвращает название должности в заданном падеже. Если падеж не найден, то возвращается строка для ручного заполнения
     * @author Алексеев К.С.
     */
    public static String getEmployeePostCaseTitle(EmployeePost employeePost, String caseTitle) {
        Post post = employeePost.getPostRelation().getPostBoundedWithQGandQL().getPost();
        switch (caseTitle) {
            case InflectorVariantCodes.RU_NOMINATIVE: {
                return post.getNominativeCaseTitle() == null ? "____________" : post.getNominativeCaseTitle();
            }
            case InflectorVariantCodes.RU_GENITIVE: {
                return post.getGenitiveCaseTitle() == null ? "____________" : post.getGenitiveCaseTitle();
            }
            case InflectorVariantCodes.RU_DATIVE: {
                return post.getDativeCaseTitle() == null ? "____________" : post.getDativeCaseTitle();
            }
            case InflectorVariantCodes.RU_ACCUSATIVE: {
                return post.getAccusativeCaseTitle() == null ? "____________" : post.getAccusativeCaseTitle();
            }
            case InflectorVariantCodes.RU_INSTRUMENTAL: {
                return post.getInstrumentalCaseTitle() == null ? "____________" : post.getInstrumentalCaseTitle();
            }
            case InflectorVariantCodes.RU_PREPOSITION: {
                return post.getPrepositionalCaseTitle() == null ? "____________" : post.getPrepositionalCaseTitle();
            }
            default:
                return "____________";
        }
    }

    /**
     * Метод возвращает email студента или null, если email не был определен
     *
     * @param studentId ID Студента
     * @return Email студента или null, если email не был определен
     */
    public static String getStudentEmail(Long studentId) {
        List<String> emails = DataAccessServices.dao().getList(
                new DQLSelectBuilder().fromEntity(Student.class, "s")
                        .column(property(Student.person().contactData().email().fromAlias("s")))
                        .where(eq(property(Student.id().fromAlias("s")), value(studentId)))
        );

        return (emails != null && !emails.isEmpty() && !emails.get(0).isBlank()) ? emails.get(0) : null;
    }

    /**
     * Метод возвращает true если debug.mode = true, иначе false
     *
     * @return Возвращает true если debug.mode = true, иначе false
     * @author Алексеев К.С.
     */
    public static boolean checkDebugMode() {
        String debugStr = ApplicationRuntime.getProperty("debug.mode");
        return Boolean.parseBoolean(debugStr != null ? debugStr : "false");
    }

}
